@@ -73,9 +73,6 @@ calculate_b_se_tval <- function(X_or_qr_decomp_of_X,
   if (fitted_only) {
     out1 <- fitted_values
   } else {
-    if (any(is.na(beta_hat))) {
-      warning("collinearity detected in ", name, "-model.\n", call. = FALSE)
-    }
     # Compute residual sum of squares (RSS)
     RSS <- colSums(residuals ^ 2)
     # Estimate variance of the errors
@@ -83,15 +80,14 @@ calculate_b_se_tval <- function(X_or_qr_decomp_of_X,
     p <- QR$rank
     sigma_hat_sq <- RSS / (n - p - 1) 
     # Calculate variance-covariance matrix of the estimated regression coefficients
-    diagXtX_inv <- diag(chol2inv(QR$qr, size = p))
-    naBetaHat <- apply(X = beta_hat, MARGIN = 1, FUN = function(x) all(is.na(x)))
-    if (sum(naBetaHat) > 0) {
-      diagXtX_inv <- c(diagXtX_inv, rep(NA, times = sum(naBetaHat)))
-      names(diagXtX_inv) <- c(colnames(QR$qr)[!naBetaHat], 
-                              rownames(beta_hat)[naBetaHat])
+    if (any(is.na(beta_hat))) {
+      warning("Collinearity detected in ", name, "-model.\n", 
+              "VIF and t-ratio's not available (NA).\n", call. = FALSE)
+      diagXtX_inv <- rep(NA, nrow(beta_hat))
     } else {
-      names(diagXtX_inv) <- colnames(QR$qr)
-    }
+	  diagXtX_inv <- diag(chol2inv(QR$qr, size = p))
+	}
+    names(diagXtX_inv) <- colnames(QR$qr)
     diagXtX_inv <- diagXtX_inv[rownames(beta_hat)]
     se <- matrix(nrow = nrow(beta_hat), ncol = length(sigma_hat_sq))
     for (i in seq_along(sigma_hat_sq)) {
